@@ -1,5 +1,9 @@
 import os
 import platform
+from datetime import datetime
+from pathlib import Path
+
+from .utils.file import Metadata
 
 try:
     import distro  # optional, only needed for Linux distros
@@ -7,21 +11,20 @@ except ImportError:
     distro = None  # type: ignore
 
 from .config import SolveigConfig
-from .schema.message import *
+from .schema.message import LLMMessage, MessageHistory, UserMessage
+
+# TODO: Make conversation examples dynamic rather than hardcoded
+from .schema.requirements import CommandRequirement, ReadRequirement
+from .schema.results import CommandResult, ReadResult
 
 SYSTEM_PROMPT = """
 You are an AI assisting a user with whatever issues they may have with their computer.
 Your goal is to be as helpful to the user as possible, and leverage the resources their computer offers to solve their problems.
 Always try to answer the user's question, no matter how redundant it may seem.
 
-You may request any of the following file operations that you think is necessary:
-- read(path, only_read_metadata): reads a file or directory. If it's a file, you can choose to read the metadata only, or the contents+metadata.
-- write(path, is_directory, content=null): creates a new file or directory, or updates an existing file. If it's a file, you may provide content to write.
-- copy(source_path, destination_path): copies a file or directory.
-- move(source_path, destination_path): moves a file or directory.
+You may request any of the following operations that you think are necessary:
+{CAPABILITIES_LIST}
 Any time you request any filesystem operations from the user, always explain why they're necessary.
-
-You may also request to run certain commands and inspect their output if you think it will help you solve user's issue.
 Any time you ask the user to execute anything, always explain why you need it, what each flag does, what you expect it to do and what the expected output is.
 Put the safety and integrity of user's system above everything else, do not suggest dangerous/destructive commands unless it's absolutely necessary.
 
@@ -29,7 +32,7 @@ The user will analyze your requirements and accept or deny each, then provide th
 You will then analyze their response and respond to it, asking for more requirements if necessary.
 You will continue this process until the user's issue is solved.
 Request as few requirements as necessary to obtain what is needed. For example, don't ask for a file and also a command to read that file.
-Prioritize asking for direct read/write requests over running commands file since it's safer for the user.
+Prioritize asking for direct file requests over running commands for those files since it's safer for the user.
 Use commands only when necessary for access or performance reasons.
 If you believe your solution will require multiple steps in sequence, ask only for what is necessary at the current stage.
 
@@ -61,7 +64,7 @@ script_chat = MessageHistory(system_prompt="")
 CONVERSATION_EXAMPLES.append(script_chat)
 script_chat.add_message(UserMessage(comment="What does the script on ~/run.sh do?"))
 file_req1 = ReadRequirement(
-    only_read_metadata=True,
+    metadata_only=True,
     path="~/run.sh",
     comment="To check what this script does, I need to read the contents of run.sh.",
 )
@@ -74,12 +77,19 @@ script_chat.add_message(
         results=[
             ReadResult(
                 requirement=file_req1,
-                metadata={
-                    "path": "/home/user/run.sh",
-                    "size": 101,
-                    "mtime": "Thu Jul 17 02:54:43 2025",
-                    "is_directory": False,
-                },
+                path=file_req1.path,
+                metadata=Metadata(
+                    owner_name="user",
+                    group_name="user",
+                    path=Path("/home/user/run.sh"),
+                    size=101,
+                    modified_time=int(
+                        datetime.fromisoformat("2025-07-17T02:54:43").timestamp()
+                    ),
+                    is_directory=False,
+                    is_readable=True,
+                    is_writable=True,
+                ),
                 accepted=True,
                 content="""
 #!/usr/bin/env bash
@@ -123,12 +133,12 @@ cmd_req_disk_usage = CommandRequirement(
 file_req_large_dirs = ReadRequirement(
     comment="If you also want to see which directories are largest in your home folder",
     path="~",
-    only_read_metadata=True,
+    metadata_only=True,
 )
 file_req_log = ReadRequirement(
     comment="I need to access the log to analyze it",
     path="~/Documents/my_app.log",
-    only_read_metadata=False,
+    metadata_only=False,
 )
 multiple_issues_chat.add_message(
     LLMMessage(
@@ -147,6 +157,7 @@ multiple_issues_chat.add_message(
         results=[
             CommandResult(
                 requirement=cmd_req_cpu_usage,
+                command=cmd_req_cpu_usage.command,
                 accepted=True,
                 stdout="""
 USER         PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
@@ -159,6 +170,7 @@ root        1203 18.4  0.9 255280  72012 ?       Ssl  07:45   7:03 /usr/lib/Xorg
             ),
             CommandResult(
                 requirement=cmd_req_disk_usage,
+                command=cmd_req_disk_usage.command,
                 accepted=True,
                 stdout="""
 Filesystem      Size  Used Avail Use% Mounted on
@@ -172,16 +184,27 @@ tmpfs           784M   48K  784M   1% /run/user/1000
 """,
                 success=True,
             ),
-            ReadResult(requirement=file_req_large_dirs, accepted=False),
+            ReadResult(
+                requirement=file_req_large_dirs,
+                path=file_req_large_dirs.path,
+                accepted=False,
+            ),
             ReadResult(
                 requirement=file_req_log,
+                path=file_req_log.path,
                 accepted=True,
-                metadata={
-                    "path": "/home/user/Documents/my_app.log",
-                    "size": 11180,
-                    "mtime": "Wed Jul 16 12:59:44 2025",
-                    "is_directory": False,
-                },
+                metadata=Metadata(
+                    owner_name="user",
+                    group_name="user",
+                    path=Path("/home/user/Documents/my_app.log"),
+                    size=11180,
+                    modified_time=int(
+                        datetime.fromisoformat("2025-07-16T12:59:44").timestamp()
+                    ),
+                    is_directory=False,
+                    is_readable=True,
+                    is_writable=True,
+                ),
                 content="""
 2025-07-16 09:12:03 INFO  [app] Starting web server on port 8080
 2025-07-16 09:12:04 INFO  [db] Connection established to postgres://localhost:5432/mydb
@@ -226,8 +249,29 @@ def get_basic_os_info(exclude_username=False):
     return info
 
 
+def get_available_capabilities() -> str:
+    """Generate capabilities list from currently filtered requirements."""
+    # Get ALL active requirements from the unified registry (core + plugins)
+    try:
+        from solveig.plugins.schema.requirements import REQUIREMENTS
+
+        active_requirements = list(REQUIREMENTS.registered.values())
+    except ImportError:
+        # Fallback if registry not available
+        active_requirements = []
+
+    return "\n".join(
+        f"- {req_class.get_description()}" for req_class in active_requirements
+    )
+
+
 def get_system_prompt(config: SolveigConfig):
-    system_prompt = SYSTEM_PROMPT.strip()
+    # Generate dynamic capabilities list
+    capabilities_list = get_available_capabilities()
+    system_prompt = SYSTEM_PROMPT.strip().replace(
+        "{CAPABILITIES_LIST}", capabilities_list
+    )
+
     if config.add_os_info:
         os_info = get_basic_os_info(config.exclude_username)
         system_prompt = (
